// Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2434 $
// $LastChangedDate:: 2022-01-27 14:59:32 +0100#$
//
// Function to create script to activate X2C installation.
// Parameters:
// X2CRoot - X2C root directory

function [] = createActivationScript(X2CRoot)

// name of activation script
scriptName = "activateScilabX2C.bat"

// get file separator
fs = filesep();

// get X2C version
verX2C = "local";
try
    fd = mopen(X2CRoot + fs + "VersionInfo.txt", "rt");
    while ~meof(fd) do
       line = mgetl(fd, 1);
       if ~isempty(strindex(line, "Version=")) then
           verX2C = strsubst(line, "Version=", "");
       end
    end
    mclose(fd);
catch
    // error during fetching of version information -> use default version
end

// get scilab home directory
scilabDir = pathconvert(SCIHOME);

// copy Scilab startup file
copyfile(scilabDir + fs + "scilab.ini", scilabDir + fs + "scilab.ini_v" + verX2C);

mprintf("Creating activation script...");
try
    // open/create file
    fileID = mopen(X2CRoot + fs + scriptName, "wt");
    
    // print header
    time = datevec(datenum());
    mfprintf(fileID, ":: Script to activate/deactivate X2C\n");
    mfprintf(fileID, ":: no input argument -> activate X2C\n");
    mfprintf(fileID, ":: any argument -> deactivate X2C\n");
    mfprintf(fileID, ":: '"purge'" as input argument -> deactivate X2C and remove backup files\n");
    mfprintf(fileID, ":: This file was generated by %s on %02d-%02d-%04d %02d:%02d\n\n", "createActivationScript.sci", time(3), time(2), time(1), time(4), time(5));
    
    mfprintf(fileID, "@ECHO off\n");
    mfprintf(fileID, "\n");
    mfprintf(fileID, "IF %%1.==. GOTO activate\n");
    mfprintf(fileID, "    :: input argument given -> deactivate X2C\n");
    mfprintf(fileID, "    ECHO Clearing environment variable...\n");
    mfprintf(fileID, "    SETX X2C_Root '"'"\n");
    mfprintf(fileID, "    REG delete HKCU\\Environment /F /V X2C_Root\n");
    mfprintf(fileID, "\n");
    mfprintf(fileID, "    ECHO Removing Scilab startup file...\n");
    mfprintf(fileID, "    DEL '"%sscilab.ini'"\n", scilabDir);
    mfprintf(fileID, "\n");
    mfprintf(fileID, "    IF %%1=='"purge'" (\n");
    mfprintf(fileID, "        ECHO Removing Scilab backup startup file...\n");
    mfprintf(fileID, "        DEL '"%sscilab.ini_v%s'"\n", scilabDir, verX2C);
    mfprintf(fileID, "    )\n");
    mfprintf(fileID, "    GOTO end\n");
    mfprintf(fileID, "\n");
    mfprintf(fileID, ":activate\n");
    mfprintf(fileID, ":: no input argument given -> activate X2C\n");
    mfprintf(fileID, "    ECHO Setting environment variable...\n");
    mfprintf(fileID, "    SETX X2C_Root '"%s%s%s'"\n", X2CRoot, fs, fs);
    mfprintf(fileID, "\n");
    mfprintf(fileID, "    ECHO Copying Scilab startup file...\n");  
    mfprintf(fileID, "    COPY '"%sscilab.ini_v%s'" '"%sscilab.ini'"\n", scilabDir, verX2C, scilabDir);
    mfprintf(fileID, "\n");
    mfprintf(fileID, ":end\n");
    
    // close file
    mclose(fileID);
    
    mprintf("DONE\n");
    
catch
    // some error creating the file occured
    mclose(fileID);
    mprintf("FAILED\n");
    mprintf("ERROR: %s\n\n", lasterror());
end


endfunction

